/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#ifdef SYSV
#define ISSOCKET(mode)	(((mode)&S_IFMT) == S_IFIFO)
#else /* BSD */
#define ISSOCKET(mode)	(((mode)&S_IFMT) == S_IFSOCK)
#endif

int
mksocket(char *name)
{
	int fd, len;
	struct sockaddr_un addr;
	struct stat sbuf;

	/*
	 * If the length of the socket address is
	 * too large, fail the request.
	 */
	if (strlen(name) > sizeof(addr.sun_path)) {
		errno = ENAMETOOLONG;
		return(-1);
	}
	if (stat(name, &sbuf) < 0) {
		/*
		 * If stat fails because the file is missing,
		 * ignore the error (we're about to create it).
		 */
		if (errno == ENOENT)
			errno = 0;
		else
			return(-1);
	} else {
		/*
		 * If the file exists and it is a socket, then
		 * unlink it.  Otherwise the name is in use.
		 */
		if (ISSOCKET(sbuf.st_mode)) {
			if (unlink(name) < 0)
				return(-1);
		} else {
			errno = EBUSY;
			return(-1);
		}
	}

	/*
	 * Create a UNIX domain datagram socket.
	 */
	if ((fd = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0)
		return(-1);

	/*
	 * Create and bind the address.
	 */
	addr.sun_family = AF_UNIX;
	strcpy(addr.sun_path, name);
	len = sizeof(addr.sun_family) + strlen(name);
	if (bind(fd, (struct sockaddr *)&addr, len) < 0)
		return(-1);

	/*
	 * Return the socket endpoint.
	 */
	return(fd);
}
